"""
web2ldap plugin classes for group related attributes
"""

import w2lapp.searchform

from w2lapp.schema.syntaxes import DistinguishedName,syntax_registry


class Member(DistinguishedName):
  oid = 'Member-oid'
  desc = 'member attribute in a group entry'

syntax_registry.registerAttrType(
  Member.oid,[
    '2.5.4.31', # member
  ]
)


class MemberOf(DistinguishedName):
  oid = 'MemberOf-oid'
  desc = 'memberOf attribute in a group member entry'

  def displayValue(self,valueindex=0,commandbutton=0):
    attr_value_u = self._ls.uc_decode(self.attrValue)[0]
    r = [DistinguishedName.displayValue(self,valueindex,commandbutton)]
    r.append(self._form.applAnchor(
      'search','Members',self._sid,
      (
        ('dn',self._ls.getSearchRoot(self._ls.uc_decode(self._dn)[0])),
        ('searchform_mode','adv'),
        ('search_attr','memberOf'),
        ('search_option',w2lapp.searchform.SEARCH_OPT_IS_EQUAL),
        ('search_string',attr_value_u),
      ),
      title=u'Search all member entries of group %s' % (attr_value_u),
    ))
    return w2lapp.cnf.misc.command_link_separator.join(r)

syntax_registry.registerAttrType(
  MemberOf.oid,[
    '1.2.840.113556.1.2.102', # memberOf
  ]
)


# Register all syntax classes in this module
for symbol_name in dir():
  syntax_registry.registerSyntaxClass(eval(symbol_name))
